@@ -24,9 +24,13 @@ config.animation_rate = 1.0
 config.blink_period = 0.8
 config.draw_whitespace = false
 config.borderless = false
+config.tab_close_button = true
 
 -- Disable plugin loading setting to false the config entry
 -- of the same name.
-config.trimwhitespace = false
+config.plugins = {}
+
+config.plugins.trimwhitespace = false
+config.plugins.lineguide = false
 
 return config
