@@ -1,4 +1,5 @@
 require "core.strict"
+require "core.regex"
 local common = require "core.common"
 local config = require "core.config"
 local style = require "core.style"
@@ -35,14 +36,8 @@ local function save_session()
 end
 
 
-local function normalize_path(s)
-  local drive, path = s:match("^([a-z]):([/\\].*)")
-  return drive and drive:upper() .. ":" .. path or s
-end
-
-
 local function update_recents_project(action, dir_path_abs)
-  local dirname = normalize_path(dir_path_abs)
+  local dirname = common.normalize_path(dir_path_abs)
   if not dirname then return end
   local recents = core.recent_projects
   local n = #recents
@@ -69,7 +64,7 @@ function core.set_project_dir(new_dir, change_project_fn)
   local chdir_ok = pcall(system.chdir, new_dir)
   if chdir_ok then
     if change_project_fn then change_project_fn() end
-    core.project_dir = normalize_path(new_dir)
+    core.project_dir = common.normalize_path(new_dir)
     core.project_directories = {}
     core.add_project_directory(new_dir)
     core.project_files = {}
@@ -285,25 +280,12 @@ end
 -- create a directory using mkdir but may need to create the parent
 -- directories as well.
 local function create_user_directory()
-  local dirname_create = USERDIR
-  local basedir
-  local subdirs = {}
-  while dirname_create and dirname_create ~= "" do
-    local success_mkdir = system.mkdir(dirname_create)
-    if success_mkdir then break end
-    dirname_create, basedir = dirname_create:match("(.*)[/\\](.+)$")
-    if basedir then
-      subdirs[#subdirs + 1] = basedir
-    end
-  end
-  for _, dirname in ipairs(subdirs) do
-    dirname_create = dirname_create .. '/' .. dirname
-    if not system.mkdir(dirname_create) then
-      error("cannot create directory: \"" .. dirname_create .. "\"")
-    end
+  local success, err = common.mkdirp(USERDIR)
+  if not success then
+    error("cannot create directory \"" .. USERDIR .. "\": " .. err)
   end
   for _, modname in ipairs {'plugins', 'colors', 'fonts'} do
-    local subdirname = dirname_create .. '/' .. modname
+    local subdirname = USERDIR .. PATHSEP .. modname
     if not system.mkdir(subdirname) then
       error("cannot create directory: \"" .. subdirname .. "\"")
     end
@@ -360,11 +342,11 @@ local style = require "core.style"
 
 -- enable or disable plugin loading setting config entries:
 
--- enable trimwhitespace, otherwise it is disable by default:
--- config.trimwhitespace = true
+-- enable plugins.trimwhitespace, otherwise it is disable by default:
+-- config.plugins.trimwhitespace = true
 --
 -- disable detectindent, otherwise it is enabled by default
--- config.detectindent = false
+-- config.plugins.detectindent = false
 ]])
   init_file:close()
 end
@@ -391,7 +373,7 @@ function core.add_project_directory(path)
   -- top directories has a file-like "item" but the item.filename
   -- will be simply the name of the directory, without its path.
   -- The field item.topdir will identify it as a top level directory.
-  path = normalize_path(path)
+  path = common.normalize_path(path)
   table.insert(core.project_directories, {
     name = path,
     item = {filename = common.basename(path), type = "dir", topdir = true},
@@ -582,10 +564,10 @@ function core.init()
 end
 
 
-function core.confirm_close_all(close_fn, ...)
+function core.confirm_close_docs(docs, close_fn, ...)
   local dirty_count = 0
   local dirty_name
-  for _, doc in ipairs(core.docs) do
+  for _, doc in ipairs(docs or core.docs) do
     if doc:is_dirty() then
       dirty_count = dirty_count + 1
       dirty_name = doc:get_name()
@@ -638,24 +620,6 @@ do
 end
 
 
--- DEPRECATED function
-core.doc_save_hooks = {}
-function core.add_save_hook(fn)
-  core.error("The function core.add_save_hook is deprecated." ..
-    " Modules should now directly override the Doc:save function.")
-  core.doc_save_hooks[#core.doc_save_hooks + 1] = fn
-end
-
-
--- DEPRECATED function
-function core.on_doc_save(filename)
-  -- for backward compatibility in modules. Hooks are deprecated, the function Doc:save
-  -- should be directly overidded.
-  for _, hook in ipairs(core.doc_save_hooks) do
-    hook(filename)
-  end
-end
-
 local function quit_with_function(quit_fn, force)
   if force then
     delete_temp_files()
@@ -663,7 +627,7 @@ local function quit_with_function(quit_fn, force)
     save_session()
     quit_fn()
   else
-    core.confirm_close_all(quit_with_function, quit_fn, true)
+    core.confirm_close_docs(core.docs, quit_with_function, quit_fn, true)
   end
 end
 
@@ -682,8 +646,8 @@ local function check_plugin_version(filename)
   if info ~= nil and info.type == "dir" then
     filename = filename .. "/init.lua"
     info = system.get_file_info(filename)
-    if not info then return true end
   end
+  if not info or not filename:match("%.lua$") then return false end
   local f = io.open(filename, "r")
   if not f then return false end
   local version_match = false
@@ -703,7 +667,7 @@ local function check_plugin_version(filename)
     end
   end
   f:close()
-  return version_match
+  return true, version_match
 end
 
 
@@ -713,24 +677,27 @@ function core.load_plugins()
     userdir = {dir = USERDIR, plugins = {}},
     datadir = {dir = DATADIR, plugins = {}},
   }
-  for _, root_dir in ipairs {USERDIR, DATADIR} do
+  local files = {}
+  for _, root_dir in ipairs {DATADIR, USERDIR} do
     local plugin_dir = root_dir .. "/plugins"
-    local files = system.list_dir(plugin_dir)
-    for _, filename in ipairs(files or {}) do
-      local basename = filename:match("(.-)%.lua$") or filename
-      local version_match = check_plugin_version(plugin_dir .. '/' .. filename)
-      if not version_match then
-        core.log_quiet("Version mismatch for plugin %q from %s", basename, plugin_dir)
-        local ls = refused_list[root_dir == USERDIR and 'userdir' or 'datadir'].plugins
-        ls[#ls + 1] = filename
-      end
-      if version_match and config[basename] ~= false then
-        local modname = "plugins." .. basename
-        local ok = core.try(require, modname)
-        if ok then core.log_quiet("Loaded plugin %q from %s", basename, plugin_dir) end
-        if not ok then
-          no_errors = false
-        end
+    for _, filename in ipairs(system.list_dir(plugin_dir) or {}) do
+      files[filename] = plugin_dir -- user plugins will always replace system plugins
+    end
+  end
+
+  for filename, plugin_dir in pairs(files) do
+    local basename = filename:match("(.-)%.lua$") or filename
+    local version_match = check_plugin_version(plugin_dir .. '/' .. filename)
+    if not version_match then
+      core.log_quiet("Version mismatch for plugin %q from %s", basename, plugin_dir)
+      local list = refused_list[plugin_dir:find(USERDIR) == 1 and 'userdir' or 'datadir'].plugins
+      table.insert(list, filename)
+    end
+    if version_match and config.plugins[basename] ~= false then
+      local ok = core.try(require, "plugins." .. basename)
+      if ok then core.log_quiet("Loaded plugin %q from %s", basename, plugin_dir) end
+      if not ok then
+        no_errors = false
       end
     end
   end
@@ -827,10 +794,30 @@ function core.normalize_to_project_dir(filename)
 end
 
 
+-- The function below works like system.absolute_path except it
+-- doesn't fail if the file does not exist. We consider that the
+-- current dir is core.project_dir so relative filename are considered
+-- to be in core.project_dir.
+-- Please note that .. or . in the filename are not taken into account.
+-- This function should get only filenames normalized using
+-- common.normalize_path function.
+function core.project_absolute_path(filename)
+  if filename:match('^%a:\\') or filename:find('/', 1, true) then
+    return filename
+  else
+    return core.project_dir .. PATHSEP .. filename
+  end
+end
+
+
 function core.open_doc(filename)
+  local new_file = not filename or not system.get_file_info(filename)
+  local abs_filename
   if filename then
+    -- normalize filename and set absolute filename then
     -- try to find existing doc for filename
-    local abs_filename = system.absolute_path(filename)
+    filename = core.normalize_to_project_dir(filename)
+    abs_filename = core.project_absolute_path(filename)
     for _, doc in ipairs(core.docs) do
       if doc.abs_filename and abs_filename == doc.abs_filename then
         return doc
@@ -838,8 +825,7 @@ function core.open_doc(filename)
     end
   end
   -- no existing doc for filename; create new
-  filename = core.normalize_to_project_dir(filename)
-  local doc = Doc(filename)
+  local doc = Doc(filename, abs_filename, new_file)
   table.insert(core.docs, doc)
   core.log_quiet(filename and "Opened doc \"%s\"" or "Opened new doc", filename)
   return doc
@@ -984,7 +970,7 @@ function core.step()
     local doc = core.docs[i]
     if #core.get_views_referencing_doc(doc) == 0 then
       table.remove(core.docs, i)
-      core.log_quiet("Closed doc \"%s\"", doc:get_name())
+      doc:on_close()
     end
   end
 
@@ -1074,6 +1060,11 @@ function core.blink_reset()
 end
 
 
+function core.request_cursor(value)
+  core.cursor_change_req = value
+end
+
+
 function core.on_error(err)
   -- write error to file
   local fp = io.open(USERDIR .. "/error.txt", "wb")
