@@ -129,36 +129,41 @@ function Node:split(dir, view, locked, resizable)
   return self.b
 end
 
-
-function Node:close_view(root, view)
-  local new_active_view = view == self.active_view
-  local do_close = function()
-    if #self.views > 1 then
-      local idx = self:get_view_idx(view)
-      table.remove(self.views, idx)
-      if new_active_view then
-        self:set_active_view(self.views[idx] or self.views[#self.views])
-      end
+function Node:remove_view(root, view)
+  if #self.views > 1 then
+    local idx = self:get_view_idx(view)
+    if idx < self.tab_offset then
+      self.tab_offset = self.tab_offset - 1
+    end
+    table.remove(self.views, idx)
+    if self.active_view == view then
+      self:set_active_view(self.views[idx] or self.views[#self.views])
+    end
+  else
+    local parent = self:get_parent_node(root)
+    local is_a = (parent.a == self)
+    local other = parent[is_a and "b" or "a"]
+    if other:get_locked_size() then
+      self.views = {}
+      self:add_view(EmptyView())
     else
-      local parent = self:get_parent_node(root)
-      local is_a = (parent.a == self)
-      local other = parent[is_a and "b" or "a"]
-      if other:get_locked_size() then
-        self.views = {}
-        self:add_view(EmptyView())
-      else
-        parent:consume(other)
-        local p = parent
-        while p.type ~= "leaf" do
-          p = p[is_a and "a" or "b"]
-        end
-        p:set_active_view(p.active_view)
-        if self.is_primary_node then
-          p.is_primary_node = true
-        end
+      parent:consume(other)
+      local p = parent
+      while p.type ~= "leaf" do
+        p = p[is_a and "a" or "b"]
+      end
+      p:set_active_view(p.active_view)
+      if self.is_primary_node then
+        p.is_primary_node = true
       end
     end
-    core.last_active_view = nil
+  end
+  core.last_active_view = nil
+end
+
+function Node:close_view(root, view)
+  local do_close = function()
+    self:remove_view(root, view)
   end
   view:try_close(do_close)
 end
@@ -169,13 +174,13 @@ function Node:close_active_view(root)
 end
 
 
-function Node:add_view(view)
+function Node:add_view(view, idx)
   assert(self.type == "leaf", "Tried to add view to non-leaf node")
   assert(not self.locked, "Tried to add view to locked node")
   if self.views[1] and self.views[1]:is(EmptyView) then
     table.remove(self.views)
   end
-  table.insert(self.views, view)
+  table.insert(self.views, idx or (#self.views + 1), view)
   self:set_active_view(view)
 end
 
@@ -235,12 +240,15 @@ end
 
 function Node:get_divider_overlapping_point(px, py)
   if self.type ~= "leaf" then
-    local p = 6
-    local x, y, w, h = self:get_divider_rect()
-    x, y = x - p, y - p
-    w, h = w + p * 2, h + p * 2
-    if px > x and py > y and px < x + w and py < y + h then
-      return self
+    local axis = self.type == "hsplit" and "x" or "y"
+    if self.a:is_resizable(axis) and self.b:is_resizable(axis) then
+      local p = 6
+      local x, y, w, h = self:get_divider_rect()
+      x, y = x - p, y - p
+      w, h = w + p * 2, h + p * 2
+      if px > x and py > y and px < x + w and py < y + h then
+        return self
+      end
     end
     return self.a:get_divider_overlapping_point(px, py)
         or self.b:get_divider_overlapping_point(px, py)
@@ -290,7 +298,7 @@ function Node:tab_hovered_update(px, py)
   if tab_index then
     local x, y, w, h = self:get_tab_rect(tab_index)
     local cx, cw = close_button_location(x, w)
-    if px >= cx and px < cx + cw and py >= y and py < y + h then
+    if px >= cx and px < cx + cw and py >= y and py < y + h and config.tab_close_button then
       self.hovered_close = tab_index
     end
   else
@@ -524,7 +532,7 @@ function Node:draw_tabs()
       renderer.draw_rect(x - ds, y, ds, h, style.divider)
     end
     local cx, cw, cspace = close_button_location(x, w)
-    local show_close_button = (view == self.active_view or i == self.hovered_tab)
+    local show_close_button = ((view == self.active_view or i == self.hovered_tab) and config.tab_close_button)
     if show_close_button then
       local close_style = self.hovered_close == i and style.text or style.dim
       common.draw_text(style.icon_font, close_style, "C", nil, cx, y, 0, h)
@@ -579,19 +587,20 @@ end
 
 function Node:is_empty()
   if self.type == "leaf" then
-    return #self.views == 0
+    return #self.views == 0 or (#self.views == 1 and self.views[1]:is(EmptyView))
   else
     return self.a:is_empty() and self.b:is_empty()
   end
 end
 
 
-function Node:close_all_docviews()
+function Node:close_all_docviews(keep_active)
   if self.type == "leaf" then
     local i = 1
     while i <= #self.views do
       local view = self.views[i]
-      if view:is(DocView) and not view:is(CommandView) then
+      if view:is(DocView) and not view:is(CommandView) and 
+        (not keep_active or view ~= self.active_view) then
         table.remove(self.views, i)
       else
         i = i + 1
@@ -601,11 +610,11 @@ function Node:close_all_docviews()
       self:add_view(EmptyView())
     end
   else
-    self.a:close_all_docviews()
-    self.b:close_all_docviews()
-    if self.a:is_empty() then
+    self.a:close_all_docviews(keep_active)
+    self.b:close_all_docviews(keep_active)
+    if self.a:is_empty() and not self.a.is_primary_node then
       self:consume(self.b)
-    elseif self.b:is_empty() then
+    elseif self.b:is_empty() and not self.b.is_primary_node then
       self:consume(self.a)
     end
   end
@@ -728,8 +737,14 @@ function RootView:open_doc(doc)
 end
 
 
-function RootView:close_all_docviews()
-  self.root_node:close_all_docviews()
+function RootView:close_all_docviews(keep_active)
+  self.root_node:close_all_docviews(keep_active)
+end
+
+
+-- Function to intercept mouse pressed events on the active view.
+-- Do nothing by default.
+function RootView.on_view_mouse_pressed(button, x, y, clicks)
 end
 
 
@@ -749,12 +764,14 @@ function RootView:on_mouse_pressed(button, x, y, clicks)
     if button == "middle" or node.hovered_close == idx then
       node:close_view(self.root_node, node.views[idx])
     else
-      self.dragged_node = idx
+      self.dragged_node = { node, idx }
       node:set_active_view(node.views[idx])
     end
   else
     core.set_active_view(node.active_view)
-    node.active_view:on_mouse_pressed(button, x, y, clicks)
+    if not self.on_view_mouse_pressed(button, x, y, clicks) then
+      node.active_view:on_mouse_pressed(button, x, y, clicks)
+    end
   end
 end
 
@@ -783,7 +800,7 @@ end
 
 function RootView:on_mouse_moved(x, y, dx, dy)
   if core.active_view == core.nag_view then
-    system.set_cursor("arrow")
+    core.request_cursor("arrow")
     core.active_view:on_mouse_moved(x, y, dx, dy)
     return
   end
@@ -808,22 +825,31 @@ function RootView:on_mouse_moved(x, y, dx, dy)
   local div = self.root_node:get_divider_overlapping_point(x, y)
   local tab_index = node and node:get_tab_overlapping_point(x, y)
   if node and node:get_scroll_button_index(x, y) then
-    system.set_cursor("arrow")
+    core.request_cursor("arrow")
   elseif div then
-    local axis = (div.type == "hsplit" and "x" or "y")
-    if div.a:is_resizable(axis) and div.b:is_resizable(axis) then
-      system.set_cursor(div.type == "hsplit" and "sizeh" or "sizev")
-    end
+    core.request_cursor(div.type == "hsplit" and "sizeh" or "sizev")
   elseif tab_index then
-    system.set_cursor("arrow")
-    if self.dragged_node and self.dragged_node ~= tab_index then
-      local tab = node.views[self.dragged_node]
-      table.remove(node.views, self.dragged_node)
-      table.insert(node.views, tab_index, tab)
-      self.dragged_node = tab_index
-    end
-  else
-    system.set_cursor(node.active_view.cursor)
+    core.request_cursor("arrow")
+  elseif node then
+    core.request_cursor(node.active_view.cursor)
+  end
+  if node and self.dragged_node and (self.dragged_node[1] ~= node or (tab_index and self.dragged_node[2] ~= tab_index))
+    and node.type == "leaf" and #node.views > 0 and node.views[1]:is(DocView) then
+      local tab = self.dragged_node[1].views[self.dragged_node[2]]
+      if self.dragged_node[1] ~= node then
+        for i, v in ipairs(node.views) do if v.doc == tab.doc then tab = nil break end end
+        if tab then
+          self.dragged_node[1]:remove_view(self.root_node, tab)
+          node:add_view(tab, tab_index)
+          self.root_node:update_layout()
+          self.dragged_node = { node, tab_index or #node.views }
+          core.redraw = true
+        end
+      else
+        table.remove(self.dragged_node[1].views, self.dragged_node[2])
+        table.insert(node.views, tab_index, tab)
+        self.dragged_node = { node, tab_index }
+      end
   end
 end
 
@@ -858,6 +884,10 @@ function RootView:draw()
     local t = table.remove(self.deferred_draws)
     t.fn(table.unpack(t))
   end
+  if core.cursor_change_req then
+    system.set_cursor(core.cursor_change_req)
+    core.cursor_change_req = nil
+  end
 end
 
 
